<?php

namespace Fundamento;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class Padding_Control {

	private $presets;
	private $preset_element;

	private $first_section = true;

	public function __construct( $presets ) {
		$this->presets = $presets;
		$this->preset_element = $presets[0]['element'];
		$this->init();
	}

	function init() {

		$presets = $this->presets;
		$preset_element = $this->preset_element;

		add_action('elementor/element/after_section_start', function( $element, $section_id, $args ) use ($presets, $preset_element) {
			if(!$this->first_section) return;
		
			if ($preset_element === $element->get_name() && $section_id !== 'element_skin' && $section_id !== 'document_settings') {
				
				$this->first_section = false;
        
        $default_option = '';

				$options = [];
        $selectors_dictionary = [];

				foreach($presets as $s) {
          $s_slug = str_replace(" ", "-", strtolower($s['name']));

          $selectors_dictionary[$s_slug] = $s['padding'];

					if(isset($s['is_default']) && $s['is_default'] === true) {
						$default_option = $s_slug;
						$options[$s_slug] = $s['name'] . ' (default)';
						continue;
					}
					$options[$s_slug] = $s['name'];
				}
				
				if($default_option === '') {
					$options[$default_option] = 'Default';
				}

        $selectors_map = [
          'column' => '{{WRAPPER}} > .elementor-widget-wrap',
        ];       

        $top_sdict = ['' => ''];
        $bottom_sdict = ['' => ''];
        $left_sdict = ['' => ''];
        $right_sdict = ['' => ''];

        foreach($selectors_dictionary as $s => $value) {
					if(\Elementor\Plugin::$instance->experiments->is_feature_active( 'container' )) {
						$top_sdict[$s] = '--padding-top: '.$value.';';
						$bottom_sdict[$s] = '--padding-bottom: '.$value.';';
						$left_sdict[$s] = '--padding-left: '.$value.';';
						$right_sdict[$s] = '--padding-right: '.$value.';';
					} else {
						$top_sdict[$s] = 'padding-top: '.$value.';';
						$bottom_sdict[$s] = 'padding-bottom: '.$value.';';
						$left_sdict[$s] = 'padding-left: '.$value.';';
						$right_sdict[$s] = 'padding-right: '.$value.';';
					}
        }

				$element->add_responsive_control(
					'top_preset_options',
					[
						'label' => __('Top Padding', 'fundamento'),
						'type' => \Elementor\Controls_Manager::SELECT,
						'devices' => ['desktop', 'tablet', 'mobile'],
						'desktop_default' => $default_option,
						'options' => $options,
            'selectors_dictionary' => $top_sdict,
            'selectors' => [
              isset($selectors_map[$preset_element]) ? $selectors_map[$preset_element] : '{{WRAPPER}}' => '{{VALUE}}',
            ]
					]
				);
        $element->add_responsive_control(
					'bottom_preset_options',
					[
						'label' => __('Bottom Padding', 'fundamento'),
						'type' => \Elementor\Controls_Manager::SELECT,
						'devices' => ['desktop', 'tablet', 'mobile'],
						'desktop_default' => $default_option,
						'options' => $options,
            'selectors_dictionary' => $bottom_sdict,
            'selectors' => [
              isset($selectors_map[$preset_element]) ? $selectors_map[$preset_element] : '{{WRAPPER}}' => '{{VALUE}}',
            ]
					]
				);
        $element->add_responsive_control(
					'left_preset_options',
					[
						'label' => __('Left Padding', 'fundamento'),
						'type' => \Elementor\Controls_Manager::SELECT,
						'devices' => ['desktop', 'tablet', 'mobile'],
						'desktop_default' => $default_option,
						'options' => $options,
            'selectors_dictionary' => $left_sdict,
            'selectors' => [
              isset($selectors_map[$preset_element]) ? $selectors_map[$preset_element] : '{{WRAPPER}}' => '{{VALUE}}',
            ]
					]
				);
				$element->add_responsive_control(
					'right_preset_options',
					[
						'label' => __('Right Padding', 'fundamento'),
						'type' => \Elementor\Controls_Manager::SELECT,
						'devices' => ['desktop', 'tablet', 'mobile'],
						'desktop_default' => $default_option,
						'options' => $options,
            'selectors_dictionary' => $right_sdict,
            'selectors' => [
              isset($selectors_map[$preset_element]) ? $selectors_map[$preset_element] : '{{WRAPPER}}' => '{{VALUE}}',
            ]
					]
				);
		
			}
		
		}, 10, 3 );
	}

}
