<?php

namespace Fundamento;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class Skin_Control {

	private $skins;
	private $skin_element;

	private $registered_stylesheets = [];
	private $enqueued_stylesheets = [];

	private $registered_scripts = [];
	private $enqueued_scripts = [];

	private $first_section = true;

	public function __construct( $skins ) {
		$this->skins = $skins;
		$this->skin_element = $skins[0]['element'];
		$this->init();
	}

	function init() {

		$skins = $this->skins;
		$skin_element = $this->skin_element;

		foreach($skins as $skin) {
			$css_handle = $skin_element.'-'.str_replace(" ", "-", strtolower($skin['name'])).'-styles';
			$js_handle = $skin_element.'-'.str_replace(" ", "-", strtolower($skin['name'])).'-script';

			if(!isset($this->registered_stylesheets[$css_handle]) && isset($skin['css'])) {
				wp_register_style($css_handle, $skin['css']);
				$this->registered_stylesheets[$css_handle] = $skin['css'];
			}
			if(!isset($this->registered_scripts[$js_handle]) && isset($skin['js'])) {
				wp_register_script($js_handle, $skin['js']);
				$this->registered_scripts[$js_handle] = $skin['js'];
			}
		}
		
		// "preview" is the page editor
		add_action('elementor/preview/enqueue_styles', function() use ($skins, $skin_element) {
			foreach($skins as $skin) {
				$css_handle = $skin_element.'-'.str_replace(" ", "-", strtolower($skin['name'])).'-styles';
				$js_handle = $skin_element.'-'.str_replace(" ", "-", strtolower($skin['name'])).'-script';

				if(isset($this->registered_stylesheets[$css_handle])) {
					wp_enqueue_style($css_handle);
					$this->enqueued_stylesheets[$css_handle] = $this->registered_stylesheets[$css_handle];
				}
				if(isset($this->registered_scripts[$js_handle])) {
					wp_enqueue_script($js_handle);
					$this->enqueued_scripts[$js_handle] = $this->registered_scripts[$js_handle];
				}
			}
		});

		add_action('elementor/element/after_section_start', function( $element, $section_id, $args ) use ($skins, $skin_element) {
			if(!$this->first_section) return;
		
			if ($skin_element === $element->get_name() && $section_id !== 'element_skin') {

				$this->first_section = false;

				$default_option = '';

				$options = [];

				foreach($skins as $s) {
          $s_slug = str_replace(" ", "-", strtolower($s['name']));

					if(isset($s['is_default']) && $s['is_default'] === true) {
						$default_option = $s_slug;
						$options[$s_slug] = $s['name'] . ' (default)';
						continue;
					}
					$options[$s_slug] = $s['name'];
				}
				
				if($default_option === '') {
					$options[''] = 'None (default)';
				}

				$element->add_control(
					'skin_options',
					[
						'label' => __('Custom Skin', 'fundamento'),
						'type' => \Elementor\Controls_Manager::SELECT,
						'default' => $default_option,
						'options' => $options,
            'prefix_class'  => 'skin-'.$skin_element.'-',
					]
				);
		
			}
		
		}, 10, 3 );

    // TODO: Remove duplicate code between render_content and before_render

		add_action('elementor/widget/render_content', function( $template, $widget ) use ($skin_element) {

      if ( $skin_element === $widget->get_name() ) {
        $settings = $widget->get_settings();      

				if(!isset($settings['skin_options']) || $settings['skin_options'] === '') return $template;

				$skin_class = ($settings && isset($settings['skin_options'])) ? 'skin-'.$skin_element.'-'.$settings['skin_options'] : '';

				$css_handle = $skin_element.'-'.$settings['skin_options'].'-styles';

				if(isset($this->registered_stylesheets[$css_handle])) {

					$found_css = false;
					foreach($this->enqueued_stylesheets as $handle => $url) {
						if($url === $this->registered_stylesheets[$css_handle]) {
							$found_css = true;
							break;
						}
					}

					if(!$found_css) {
						wp_enqueue_style($css_handle);
						$this->enqueued_stylesheets[$css_handle] = $this->registered_stylesheets[$css_handle];
					}
				}

				$js_handle = $skin_element.'-'.$settings['skin_options'].'-script';

				if(isset($this->registered_scripts[$js_handle])) {

					$found_js = false;
					foreach($this->enqueued_scripts as $handle => $url) {
						if($url === $this->registered_scripts[$js_handle]) {
							$found_js = true;
							break;
						}
					}

					if(!$found_js) {
						wp_enqueue_script($js_handle);
						$this->enqueued_scripts[$js_handle] = $this->registered_scripts[$js_handle];
					}
				}

			}
			return $template;

		}, 10, 2);

    if($skin_element === 'section' || $skin_element === 'column') {
      add_action('elementor/frontend/before_render', function( $element ) use ($skin_element) {
        
        if ($skin_element === $element->get_name()) {
          $settings = $element->get_settings();

          if(!isset($settings['skin_options']) || $settings['skin_options'] === '') return;

          $skin_class = ($settings && isset($settings['skin_options']) && $settings['skin_options'] !== '') ? 'skin-'.$skin_element.'-'.$settings['skin_options'] : '';
      
          $css_handle = $skin_element.'-'.$settings['skin_options'].'-styles';

          if(isset($this->registered_stylesheets[$css_handle])) {

            $found_css = false;
            foreach($this->enqueued_stylesheets as $handle => $url) {
              if($url === $this->registered_stylesheets[$css_handle]) {
                $found_css = true;
                break;
              }
            }

            if(!$found_css) {
              wp_enqueue_style($css_handle);
              $this->enqueued_stylesheets[$css_handle] = $this->registered_stylesheets[$css_handle];
            }
          }

          $js_handle = $skin_element.'-'.$settings['skin_options'].'-script';

          if(isset($this->registered_scripts[$js_handle])) {

            $found_js = false;
            foreach($this->enqueued_scripts as $handle => $url) {
              if($url === $this->registered_scripts[$js_handle]) {
                $found_js = true;
                break;
              }
            }

            if(!$found_js) {
              wp_enqueue_script($js_handle);
              $this->enqueued_scripts[$js_handle] = $this->registered_scripts[$js_handle];
            }
          }
        }
      }, 10, 1);
    }
	}

}
